module Clerk
  class Engine < ::Rails::Engine
    isolate_namespace Clerk
    engine_name "clerk"

    config.before_configuration do 
      if Rails.env.development?
        require 'dotenv'
        Dotenv.load
      end

      if Clerk.key.nil?
        raise "Clerk is installed but not configured.  Set your CLERK_KEY as an environment variable."
      end

      begin
        config_data = Clerk.api.get('/api/environment').data
        Clerk.configure do |config|
          config.host                       = config_data[:HOST]
          config.accounts_host              = config_data[:ACCOUNTS_HOST]
          config.app_host                   = config_data[:APP_HOST]
          config.cipher_key                 = config_data[:CIPHER_KEY]
          config.database_url               = config_data[:DATABASE_URL]
          config.email_address_verification = config_data[:EMAIL_ADDRESS_VERIFICATION]
          config.ngrok_authtoken            = config_data[:NGROK_AUTHTOKEN]
          config.tunnel_cert                = config_data[:TUNNEL_CERT]
          config.tunnel_key                 = config_data[:TUNNEL_KEY]          
        end
      rescue => e
        raise "Clerk could not be loaded. #{e} #{config_data}"
      end

      if defined?(Rails::Server) and Rails.env.development?
        require 'clerk/tunnel'
        Clerk::Tunnel.start!
      end
    end

    accounts_routes = {
      sign_out:             "/sign_out",
      sign_in:              "/",
      sign_up:              "/sign_up",
      verify_email_address: "/verify_email_address",
    }


    initializer 'clerk.configuration' do |app|
      app.routes.append do
        mount Clerk::Engine => "/"

        accounts_routes.each do |k, v|
          direct k do
            "#{Clerk.accounts_url}#{v}"
          end
        end
      end
    end        
  end
end
